//
//  BsMapManager.swift
//  BaiSiSMApp
//
//  Created by DAVID on 2022/12/15.
//  Copyright © 2022 www.davidhuang.com. All rights reserved.
//

import UIKit
import MapKit

class BsMapManager {
    
    // MARK: - 属性
    static let sharedInstance = BsMapManager()
    
    public let hasQqMap: Bool = UIApplication.shared.canOpenURL(URL(string: "qqmap://")!)
    
    public let hasIosaMap: Bool = UIApplication.shared.canOpenURL(URL(string: "iosamap://")!)
    
    public let hasBaiduMap: Bool = UIApplication.shared.canOpenURL(URL(string: "baidumap://")!)
    
    // 当前位置或起始位置
    var currentLocation: CLLocationCoordinate2D!
    var currentAddress: String = ""
    
    // 目标位置
    var targetLocation: CLLocationCoordinate2D!
    var targetAddress: String = ""
    
    // MARK: - 导航位置初始化
    func initNavi(currentLoc: CLLocationCoordinate2D! = nil, currentAdd: String = "", targetLoc: CLLocationCoordinate2D!, targetAdd: String) {
        if currentLoc != nil {
            currentLocation = currentLoc
        }
        
        currentAddress = currentAdd
        
        if targetLoc != nil {
            targetLocation = targetLoc
        }
        
        targetAddress = targetAdd
    }
    
    // MARK: - Apple地图导航
    func naviWithAppleMap() {
        if targetLocation == nil {
            return
        }
        
        // 起始位置
        let fromLocation = MKMapItem.forCurrentLocation()
        fromLocation.name = "我的位置"
        
        // 目标位置
        let toCoor = CLLocationCoordinate2D(latitude: targetLocation!.latitude, longitude: targetLocation!.longitude)
        let toLocation = MKMapItem(placemark: MKPlacemark(coordinate: toCoor, addressDictionary: [:]))
        toLocation.name = targetAddress
        
        // 打开地图
        MKMapItem.openMaps(with: [fromLocation, toLocation],
                           launchOptions: [MKLaunchOptionsDirectionsModeKey: MKLaunchOptionsDirectionsModeDriving, MKLaunchOptionsShowsTrafficKey: NSNumber(true)])
    }
    
    // MARK: - 百度地图导航
    func naviWithBaiduMap() {
        if targetLocation == nil {
            return
        }
        
        /// 请求格式：
        /// baidumap://map/direction?origin=34.264642646862,108.95108518068&destination=40.007623,116.360582&coord_type=bd09ll&mode=driving&src=ios.baidu.openAPIdemo
        
        var urlString: String = "baidumap://map/direction?"
        
        // 起始位置
        let origin: String = "我的位置"
        urlString = urlString + "origin=\(origin)"
        
        // 目标位置
        let destination: String = "\(targetLocation!.latitude),\(targetLocation!.longitude)"
        let name: String = targetAddress
        urlString = urlString + "&destination=name:\(name)|latlng:\(destination)"
        
        /// 可选的坐标系统：
        ///     bd09ll（百度经纬度坐标）
        ///     bd09mc（百度墨卡托坐标）
        ///     gcj02（经国测局加密的坐标）
        ///     wgs84（gps获取的原始坐标）
        urlString = urlString + "&coord_type=gcj02&mode=driving&src=ios.ftsafe.FTSmartSudentCard"
        
        urlString = urlString.addingPercentEncoding(withAllowedCharacters: CharacterSet.urlQueryAllowed)!
        
        if #available(iOS 10.0, *) {
            UIApplication.shared.open(URL(string: urlString)!, options: [:], completionHandler: nil)
        } else {
            UIApplication.shared.openURL(URL(string: urlString)!)
        }
    }
    
    // MARK: - 腾讯地图导航
    func naviWithQqMap() {
        if targetLocation == nil {
            return
        }
        
        /// 请求格式：
        /// qqmap://map/routeplan?type=drive&from=清华&fromcoord=39.994745,116.247282&to=怡和世家&tocoord=39.867192,116.493187&referer=OB4BZ-D4W3U-B7VVO-4PJWW-6TKDJ-WPB77
        /// 这里可能需要开发者key，即referer
        
        var urlString: String = "qqmap://map/routeplan?type=drive"
        let origin: String = "我的位置"
        urlString = urlString + "&from=\(origin)&fromcoord=CurrentLocation"
        
        let destination: String = "\(targetLocation!.latitude),\(targetLocation!.longitude)"
        let name: String = targetAddress
        urlString = urlString + "&to=\(name)&tocoord=\(destination)"
        
        urlString = urlString.addingPercentEncoding(withAllowedCharacters: CharacterSet.urlQueryAllowed)!
        
        if #available(iOS 10.0, *) {
            UIApplication.shared.open(URL(string: urlString)!, options: [:], completionHandler: nil)
        } else {
            UIApplication.shared.openURL(URL(string: urlString)!)
        }
    }
    
    // MARK: - 高德地图导航
    func naviWithAMap() {
        if targetLocation == nil {
            return
        }
        
        /// 请求格式：
        /// iosamap://path?sourceApplication=applicationName&sid=&slat=39.92848272&slon=116.39560823&sname=A&did=&dlat=39.98848272&dlon=116.47560823&dname=B&dev=0&t=0
        
        var urlString: String = "iosamap://path?sourceApplication=FTSmartSudentCard"
        
        let name: String = targetAddress
        urlString = urlString + "&dlat=\(targetLocation!.latitude)&dlon=\(targetLocation!.longitude)&dname=\(name)&dev=0&t=0"
        
        urlString = urlString.addingPercentEncoding(withAllowedCharacters: CharacterSet.urlQueryAllowed)!
        
        if #available(iOS 10.0, *) {
            UIApplication.shared.open(URL(string: urlString)!, options: [:], completionHandler: nil)
        } else {
            UIApplication.shared.openURL(URL(string: urlString)!)
        }
    }
}
